       TITLE 'SOCKET OWNING REGION DISPATCHER'
      *****************************************************************
      *                                                               *
      * MODULE NAME = IJMRDISP                                        *
      *                                                               *
      * DESCRIPTIVE NAME = A CICS REQUESTER DISPATCHER FOR AN SOR     *
      *                                                               *
      * STATUS = SUPPORTED BY CICS TRANSACTION SERVER 1.3 OR LATER    *
      *                                                               *
      *---------------------------------------------------------------*
      *                                                               *
      * CHANGE ACTIVITY :                                             *
      *                                                               *
      * 210202  IJM   : Created                                       *
      *                                                               *
      *---------------------------------------------------------------*
      *****************************************************************
      *             PROGRAM DESCRIPTION                               *
      *****************************************************************
      *                                                               *
      * ...                                                           *
      *****************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID.  IJMRDISP.
      *
       AUTHOR.      IBM.
      *
       DATE-COMPILED.
      *
      *REMARKS.
      *****************************************************************
      *                                                               *
      *                                                               *
      *****************************************************************
      *
       ENVIRONMENT DIVISION.
      *
       CONFIGURATION SECTION.
      *
       SPECIAL-NAMES.
      *
       DATA DIVISION.

       WORKING-STORAGE SECTION.

       01  WS-START                        PIC X(44)
           VALUE '*** IJMRDISP WORKING STORAGE STARTS HERE ***'.

       COPY IJMRCON.

      * WORKING VARIABLES
       01  PROCESS-NAME.
           03  HEADER                      PIC X(8)  VALUE 'BTS-REQH'.
           03  CLIENTIPADDR                PIC X(15).
           03  FILLER                      PIC X(13) VALUE SPACES.
       01  PROCESS-STATUS                  PIC 9(8)  COMP.
       01  PROCESS-ABCODE                  PIC X(4).
       01  ROOTID-VALUE                    PIC X(52).
       01  IPADDRL                         PIC 9(8)  COMP.
       01  INPUT-LENGTH                    PIC 9(8)  COMP.
       01  RESP-AREA                       PIC 9(8)  COMP.
      *
       LINKAGE SECTION.
      *
       PROCEDURE DIVISION.

      ******************************************************************
      *BEGIN-PROCESS                                                   *
      ******************************************************************
       BEGIN-PROCESS.

      * CONSTRUCT A NEW PROCESSNAME.

           MOVE SPACES TO CLIENTIPADDR.
           MOVE LENGTH OF CLIENTIPADDR TO IPADDRL.
           EXEC CICS EXTRACT TCPIP
                     CLIENTADDR(CLIENTIPADDR)
                     CADDRLENGTH(IPADDRL)
           END-EXEC.

      * DEFINE THE PROCESS.

           EXEC CICS DEFINE PROCESS(PROCESS-NAME)
                     PROCESSTYPE(PROCESS-TYPE)
                     TRANSID(PROCESS-TRANID)
                     PROGRAM(PROCESS-PROG)
           END-EXEC.

      * GIVE THE REQUEST TO THE PROCESS

           MOVE EIBCALEN TO INPUT-LENGTH.
           EXEC CICS PUT ACQPROCESS
                     CONTAINER(PROCESS-INPUT)
                     FROM(DFHCOMMAREA)
                     FLENGTH(INPUT-LENGTH)
           END-EXEC.

      * EXECUTE THE FIRST PHASE OF THE PROCESS LOCALLY (IN THE SOR)

           EXEC CICS LINK ACQPROCESS
                     INPUTEVENT('DFHINITIAL')
           END-EXEC.

      * CHECK THE OUTCOME OF THE FIRST PHASE

           EXEC CICS CHECK ACQPROCESS
                     COMPSTATUS(PROCESS-STATUS)
                     ABCODE(PROCESS-ABCODE)
           END-EXEC.
           IF PROCESS-STATUS = DFHVALUE(COMPLETE) THEN
             EXEC CICS RETURN
             END-EXEC.
      *    END-IF.

      * GET THE ROOT ACTIVITY ID FROM THE CONTAINER

           EXEC CICS GET ACQACTIVITY
                     CONTAINER(ROOTID-CONTAINER)
                     INTO(ROOTID-VALUE)
           END-EXEC.

      * PERFORM A SYNCPOINT TO RELEASE THE PROCESS

           EXEC CICS SYNCPOINT
           END-EXEC.

      * DPL TO THE AOR SERVER PROGRAM TO EXECUTE PHASE 2

           EXEC CICS LINK PROGRAM(AORSERVER)
                     COMMAREA(ROOTID-VALUE)
                     LENGTH(52)
                     DATALENGTH(52)
                     SYNCONRETURN
           END-EXEC.

      * REACQUIRE THE PROCESS TO CONTINUE PROCESSING IN THE SOR

           EXEC CICS ACQUIRE PROCESS(PROCESS-NAME)
                     PROCESSTYPE(PROCESS-TYPE)
           END-EXEC.

      * EXECUTE THE THIRD PHASE OF THE PROCESS LOCALLY (IN THE SOR)

           EXEC CICS LINK ACQPROCESS
                     INPUTEVENT(PHASE3)
           END-EXEC.

      * CHECK THE OUTCOME OF THE THIRD PHASE

           EXEC CICS CHECK ACQPROCESS
                     COMPSTATUS(PROCESS-STATUS)
                     ABCODE(PROCESS-ABCODE)
           END-EXEC.

      * RETURN THE OUTPUT IN THE COMMAREA
           MOVE SPACES TO DFHCOMMAREA.
           EXEC CICS GET ACQACTIVITY CONTAINER(PROCESS-OUTPUT)
                                     INTO(DFHCOMMAREA)
                                     FLENGTH(INPUT-LENGTH)
                                     RESP(RESP-AREA)
           END-EXEC.
           IF RESP-AREA = DFHRESP(LENGERR) THEN
           EXEC CICS GET ACQACTIVITY CONTAINER(PROCESS-OUTPUT)
                                     INTO(DFHCOMMAREA)
                                     FLENGTH(INPUT-LENGTH)
                                     RESP(RESP-AREA)
           END-EXEC.
      *    END-IF.
      ******************************************************************
      *DUMMY-GOBACK                                                    *
      *BECAUSE SOME COMPILERS NEED IT                                  *
      ******************************************************************
       DUMMY-GOBACK.
            GOBACK.
      *
